"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const ng = require("@angular/compiler-cli");
const path = require("path");
const log = require("../util/log");
const create_emit_callback_1 = require("./create-emit-callback");
const redirect_write_file_compiler_host_1 = require("../ts/redirect-write-file-compiler-host");
const cache_compiler_host_1 = require("../ts/cache-compiler-host");
function compileSourceFiles(tsConfig, sourcesFileCache, moduleResolutionCache, stylesheetProcessor, extraOptions, declarationDir) {
    return __awaiter(this, void 0, void 0, function* () {
        log.debug(`ngc (v${ng.VERSION.full})`);
        const tsConfigOptions = Object.assign({}, tsConfig.options, extraOptions);
        let tsCompilerHost = cache_compiler_host_1.cacheCompilerHost(tsConfigOptions, sourcesFileCache, moduleResolutionCache, stylesheetProcessor);
        if (declarationDir) {
            tsCompilerHost = redirect_write_file_compiler_host_1.redirectWriteFileCompilerHost(tsCompilerHost, tsConfigOptions.basePath, declarationDir);
        }
        // ng.CompilerHost
        const ngCompilerHost = ng.createCompilerHost({
            options: tsConfigOptions,
            tsHost: tsCompilerHost
        });
        // Don't use `ng.emit` as it doesn't output all errors.
        // https://github.com/angular/angular/issues/24024
        const result = ng.performCompilation({
            rootNames: tsConfig.rootNames,
            options: tsConfigOptions,
            host: ngCompilerHost,
            emitCallback: create_emit_callback_1.createEmitCallback(tsConfigOptions),
            emitFlags: tsConfig.emitFlags
        });
        // XX(hack): redirect the `*.metadata.json` to the correct outDir
        // @link https://github.com/angular/angular/pull/21787
        if (declarationDir) {
            const flatModuleFile = tsConfigOptions.flatModuleOutFile;
            const flatModuleFileExtension = path.extname(flatModuleFile);
            const metadataBundleFile = flatModuleFile.replace(flatModuleFileExtension, '.metadata.json');
            const metadataSrc = path.resolve(tsConfigOptions.declarationDir, metadataBundleFile);
            const metadataDest = path.resolve(declarationDir, metadataBundleFile);
            if (metadataDest !== metadataSrc && fs.existsSync(metadataSrc)) {
                fs.moveSync(metadataSrc, metadataDest, { overwrite: true });
            }
        }
        const exitCode = ng.exitCodeFromResult(result.diagnostics);
        return exitCode === 0 ? Promise.resolve() : Promise.reject(new Error(ng.formatDiagnostics(result.diagnostics)));
    });
}
exports.compileSourceFiles = compileSourceFiles;
//# sourceMappingURL=compile-source-files.js.map